=== modified file 'bzrlib/smart/__init__.py'
--- bzrlib/smart/__init__.py	2009-03-23 14:59:43 +0000
+++ bzrlib/smart/__init__.py	2011-09-22 14:35:58 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006 Canonical Ltd
+# Copyright (C) 2006,2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -33,25 +33,11 @@
 
 """
 
-# TODO: _translate_error should be on the client, not the transport because
-#     error coding is wire protocol specific.
-
 # TODO: A plain integer from query_version is too simple; should give some
 # capabilities too?
 
-# TODO: Server should probably catch exceptions within itself and send them
-# back across the network.  (But shouldn't catch KeyboardInterrupt etc)
-# Also needs to somehow report protocol errors like bad requests.  Need to
-# consider how we'll handle error reporting, e.g. if we get halfway through a
-# bulk transfer and then something goes wrong.
-
 # TODO: Make each request and response self-validatable, e.g. with checksums.
 #
-# TODO: get/put objects could be changed to gradually read back the data as it
-# comes across the network
-#
-# TODO: What should the server do if it hits an error and has to terminate?
-#
 # TODO: is it useful to allow multiple chunks in the bulk data?
 #
 # TODO: If we get an exception during transmission of bulk data we can't just
@@ -60,44 +46,6 @@
 #   chunk, that indicates it is another chunk. Then you can send an 'error'
 #   chunk as long as you finish the previous chunk.
 #
-# TODO: Clone method on Transport; should work up towards parent directory;
-# unclear how this should be stored or communicated to the server... maybe
-# just pass it on all relevant requests?
-#
-# TODO: Better name than clone() for changing between directories.  How about
-# open_dir or change_dir or chdir?
-#
-# TODO: Is it really good to have the notion of current directory within the
-# connection?  Perhaps all Transports should factor out a common connection
-# from the thing that has the directory context?
-#
-# TODO: The server that manages a connection should be quite small and retain
-# minimum state because each of the requests are supposed to be stateless.
-# Then we can write another implementation that maps to http.
-#
-# TODO: What to do when a client connection is garbage collected?  Maybe just
-# abruptly drop the connection?
-#
-# TODO: Server in some cases will need to restrict access to files outside of
-# a particular root directory.  LocalTransport doesn't do anything to stop you
-# ascending above the base directory, so we need to prevent paths
-# containing '..' in either the server or transport layers.  (Also need to
-# consider what happens if someone creates a symlink pointing outside the
-# directory tree...)
-#
-# TODO: Server should rebase absolute paths coming across the network to put
-# them under the virtual root, if one is in use.  LocalTransport currently
-# doesn't do that; if you give it an absolute path it just uses it.
-#
-# XXX: Arguments can't contain newlines or ascii; possibly we should e.g.
-# urlescape them instead.  Indeed possibly this should just literally be
-# http-over-ssh.
-#
-# TODO: Probably want some way for server commands to gradually produce body
-# data rather than passing it as a string; they could perhaps pass an
-# iterator-like callback that will gradually yield data; it probably needs a
-# close() method that will always be closed to do any necessary cleanup.
-
 
 # Promote some attributes from submodules into this namespace
 from bzrlib.smart.request import SmartServerRequestHandler

=== modified file 'bzrlib/smart/medium.py'
--- bzrlib/smart/medium.py	2011-09-22 13:01:09 +0000
+++ bzrlib/smart/medium.py	2011-09-22 14:35:58 +0000
@@ -225,8 +225,8 @@
         """Serve requests until the client disconnects."""
         # Keep a reference to stderr because the sys module's globals get set to
         # None during interpreter shutdown.
-        from bzrlib.smart import server
-        server.register_sighup_callback(id(self), self.stop)
+        from bzrlib.smart import signals
+        signals.register_on_hangup(id(self), self._stop_gracefully)
         from sys import stderr
         try:
             while not self.finished:
@@ -246,9 +246,12 @@
         except Exception, e:
             stderr.write("%s terminating on exception %s\n" % (self, e))
             raise
-        server.unregister_sighup_callback(id(self))
+        finally:
+            signals.unregister_on_hangup(id(self))
 
-    def stop(self):
+    def _stop_gracefully(self):
+        """When we finish this message, stop looking for more."""
+        trace.mutter('Stopping %s' % (self,))
         self.finished = True
 
     def _close(self):

=== modified file 'bzrlib/smart/server.py'
--- bzrlib/smart/server.py	2011-09-22 13:01:09 +0000
+++ bzrlib/smart/server.py	2011-09-22 14:35:58 +0000
@@ -30,7 +30,10 @@
 )
 from bzrlib.lazy_import import lazy_import
 lazy_import(globals(), """
-from bzrlib.smart import medium
+from bzrlib.smart import (
+    medium,
+    signals,
+    )
 from bzrlib.transport import (
     chroot,
     pathfilter,
@@ -41,28 +44,6 @@
     )
 """)
 
-import signal
-
-_on_sighup = {}
-def sighup_handler(signal_number, interrupted_frame):
-    for i, c in _on_sighup.items():
-        # TODO: ignore errors here
-        print "Calling %s: %s" % (i, c)
-        c()
-
-signal.signal(signal.SIGHUP, sighup_handler)
-
-
-def register_sighup_callback(identifier, a_callable):
-    """Register for us to call a_callable after SIGHUP."""
-    _on_sighup[identifier] = a_callable
-
-
-def unregister_sighup_callback(identifier):
-    """Remove a callback from being called during sighup."""
-    # TODO: Ignore errors
-    del _on_sighup[identifier]
-
 
 class SmartTCPServer(object):
     """Listens on a TCP socket and accepts connections from smart clients.
@@ -170,7 +151,7 @@
         self._should_terminate = True
 
     def serve(self, thread_name_suffix=''):
-        register_sighup_callback(id(self), self._stop_gracefully)
+        signals.register_on_hangup(id(self), self._stop_gracefully)
         self._should_terminate = False
         # for hooks we are letting code know that a server has started (and
         # later stopped).
@@ -202,7 +183,7 @@
                 raise
         finally:
             self._stopped.set()
-            unregister_sighup_callback(id(self))
+            signals.unregister_on_hangup(id(self))
             try:
                 # ensure the server socket is closed.
                 self._server_socket.close()

=== added file 'bzrlib/smart/signals.py'
--- bzrlib/smart/signals.py	1970-01-01 00:00:00 +0000
+++ bzrlib/smart/signals.py	2011-09-22 14:35:58 +0000
@@ -0,0 +1,78 @@
+# Copyright (C) 2011 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+
+"""Signal handling for the smart server code."""
+
+import signal
+import weakref
+
+from bzrlib import trace
+
+
+# I'm pretty sure this has to be global, since signal handling is per-process.
+_on_sighup = weakref.WeakValueDictionary()
+# TODO: Using a dict means that the order of calls is unordered. We could use a
+#       list and then do something like LIFO ordering. A dict was chosen so
+#       that you could have a key to easily remove your entry. However, you
+#       could just use the callable itself as the indexed part, and even in
+#       large cases, we shouldn't have more than 100 or so callbacks
+#       registered.
+def _sighup_handler(signal_number, interrupted_frame):
+    for ref in _on_sighup.valuerefs():
+        # TODO: ignore errors here
+        try:
+            cb = ref()
+            if cb is not None:
+                cb()
+        except KeyboardInterrupt:
+            raise
+        except Exception:
+            trace.mutter('Error occurred while running SIGHUP handlers:')
+            trace.log_exception_quietly()
+
+
+# TODO: One option, we could require install_sighup_handler to be called,
+#       before we allocate _on_sighup. And then register_on_hangup would become
+#       a no-op if install_sighup_handler was never installed. This would help
+#       us avoid creating garbage if the test suite isn't going to handle it.
+def install_sighup_handler():
+    """Setup a handler for the SIGHUP signal."""
+    signal.signal(signal.SIGHUP, _sighup_handler)
+
+
+# TODO: Should these be single-use callables? Meaning that once we've triggered
+#       SIGHUP and called them, they should auto-remove themselves? I don't
+#       think so. Callers need to clean up during shutdown anyway, so that we
+#       don't end up with lots of garbage in the _on_sighup dict. On the other
+#       hand, we made _on_sighup a WeakValueDictionary in case cleanups didn't
+#       get fired properly. Maybe we just assume we don't have to do it?
+def register_on_hangup(identifier, a_callable):
+    """Register for us to call a_callable as part of a graceful shutdown."""
+    _on_sighup[identifier] = a_callable
+
+
+def unregister_on_hangup(identifier):
+    """Remove a callback from being called during sighup."""
+    try:
+        del _on_sighup[identifier]
+    except KeyboardInterrupt:
+        raise
+    except Exception:
+        # This usually runs as a tear-down step. So we don't want to propagate
+        # most exceptions.
+        trace.mutter('Error occurred during unregister_on_hangup:')
+        trace.log_exception_quietly()
+

=== modified file 'bzrlib/tests/__init__.py'
--- bzrlib/tests/__init__.py	2011-09-15 14:05:18 +0000
+++ bzrlib/tests/__init__.py	2011-09-22 14:35:58 +0000
@@ -4077,6 +4077,7 @@
         'bzrlib.tests.test_smart',
         'bzrlib.tests.test_smart_add',
         'bzrlib.tests.test_smart_request',
+        'bzrlib.tests.test_smart_signals',
         'bzrlib.tests.test_smart_transport',
         'bzrlib.tests.test_smtp_connection',
         'bzrlib.tests.test_source',

=== added file 'bzrlib/tests/test_smart_signals.py'
--- bzrlib/tests/test_smart_signals.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/test_smart_signals.py	2011-09-22 14:35:58 +0000
@@ -0,0 +1,110 @@
+# Copyright (C) 2011 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+
+
+import gc
+import signal
+import weakref
+
+from bzrlib import tests
+from bzrlib.smart import signals
+
+
+class TestSignalHandlers(tests.TestCase):
+
+    def setUp(self):
+        super(TestSignalHandlers, self).setUp()
+        # This allows us to mutate the signal handler callbacks, but leave it
+        # 'pristine' after the test case.
+        # TODO: Arguably, this could be put into the base test.TestCase, along
+        #       with a tearDown that asserts that all the entries have been
+        #       removed properly. Global state is always a bit messy. A shame
+        #       that we need it for signal handling.
+        self.orig__on_sighup = self.overrideAttr(signals, '_on_sighup',
+                                                 weakref.WeakValueDictionary())
+
+    def test_registered_callback_gets_called(self):
+        calls = []
+        def call_me():
+            calls.append('called')
+        signals.register_on_hangup('myid', call_me)
+        signals._sighup_handler(signal.SIGHUP, None)
+        self.assertEqual(['called'], calls)
+        signals.unregister_on_hangup('myid')
+
+    def test_unregister_not_present(self):
+        # We don't want unregister to fail, since it is generally run at times
+        # that shouldn't interrupt other flow.
+        signals.unregister_on_hangup('no-such-id')
+        log = self.get_log()
+        self.assertContainsRe(log, 'Error occurred during unregister_on_hangup:')
+        self.assertContainsRe(log, '(?s)Traceback.*KeyError')
+
+    def test_failing_callback(self):
+        calls = []
+        def call_me():
+            calls.append('called')
+        def fail_me():
+            raise RuntimeError('something bad happened')
+        signals.register_on_hangup('myid', call_me)
+        signals.register_on_hangup('otherid', fail_me)
+        # _sighup_handler should call both, even though it got an exception
+        signals._sighup_handler(signal.SIGHUP, None)
+        signals.unregister_on_hangup('myid')
+        signals.unregister_on_hangup('otherid')
+        log = self.get_log()
+        self.assertContainsRe(log, '(?s)Traceback.*RuntimeError')
+        self.assertEqual(['called'], calls)
+
+    def test_unregister_during_call(self):
+        # _sighup_handler should handle if some callbacks actually remove
+        # themselves while running.
+        calls = []
+        def call_me_and_unregister():
+            signals.unregister_on_hangup('myid')
+            calls.append('called_and_unregistered')
+        def call_me():
+            calls.append('called')
+        signals.register_on_hangup('myid', call_me_and_unregister)
+        signals.register_on_hangup('other', call_me)
+        signals._sighup_handler(signal.SIGHUP, None)
+
+    def test_keyboard_interrupt_propagated(self):
+        # In case we get 'stuck' while running a hangup function, we should
+        # not suppress KeyboardInterrupt
+        def call_me_and_raise():
+            raise KeyboardInterrupt()
+        signals.register_on_hangup('myid', call_me_and_raise)
+        self.assertRaises(KeyboardInterrupt,
+                          signals._sighup_handler, signal.SIGHUP, None)
+        signals.unregister_on_hangup('myid')
+
+    def test_weak_references(self):
+        # TODO: This is probably a very-CPython-specific test
+        # Adding yourself to the callback should not make you immortal
+        # We overrideAttr during the test suite, so that we don't pollute the
+        # original dict. However, we can test that what we override matches
+        # what we are putting there.
+        self.assertIsInstance(self.orig__on_sighup,
+                              weakref.WeakValueDictionary)
+        calls = []
+        def call_me():
+            calls.append('called')
+        signals.register_on_hangup('myid', call_me)
+        del call_me
+        gc.collect()
+        signals._sighup_handler(signal.SIGHUP, None)
+        self.assertEqual([], calls)

=== modified file 'bzrlib/tests/test_smart_transport.py'
--- bzrlib/tests/test_smart_transport.py	2011-09-22 13:01:09 +0000
+++ bzrlib/tests/test_smart_transport.py	2011-09-22 14:35:58 +0000
@@ -941,7 +941,7 @@
         # _build_protocol should notice that we are stopping, and return
         # without waiting for bytes from the client.
         server, client_sock = self.create_stream_context(None)
-        server.stop()
+        server._stop_gracefully()
         self.assertIs(None, server._build_protocol())
 
     def test_socket_set_timeout(self):
@@ -990,7 +990,7 @@
         # because of a timeout, etc.
         server._timer = lambda: t
         server._client_poll_timeout = 0.1
-        server.stop()
+        server._stop_gracefully()
         server._wait_for_bytes_with_timeout(1.0)
 
     def test_socket_serve_timeout_closes_socket(self):

